local _, BigBrother = ...

_G.BigBrother = LibStub("AceAddon-3.0"):NewAddon(BigBrother, "BigBrother", "AceEvent-3.0")
BigBrother:SetDefaultModuleLibraries("AceEvent-3.0")

local L = LibStub("AceLocale-3.0"):GetLocale("BigBrother")

function BigBrother:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("BigBrotherDB", {
		profile = {
			autogroups = true,
			groups = { true, true, true, true, true, true, true, true },
			checkFlasks = true,
			checkFood = true,
			checkBuffs = true,
			checkReadyCheck = 1,
			outputDefault = "raid",
			maxOnly = true,
		}
	})
end

function BigBrother:OnEnable()
	self:RegisterEvent("READY_CHECK")
end

function BigBrother:Print(...)
	print("|cff33ff99BigBrother|r:", ...)
end

function BigBrother:GetInstanceSize()
	local _, instanceType, diff = GetInstanceInfo()
	if instanceType == "party" or instanceType == "arena" then
		return 5
	elseif instanceType == "none" or instanceType == "pvp" or diff == 9 then
		return 40
	elseif diff == 3 or diff == 5 then
		return 10
	elseif diff == 4 or diff == 6 or diff == 7 then
		return 25
	end
end

do -- BigBrother:SendChatMessageList(title, list, channel)
	local chatTypes = {
		SELF = true,
		SAY = true,
		GROUP = true,
		PARTY = true,
		GUILD = true,
		OFFICIER = true,
		RAID = true,
		RAID_WARNING = true,
		WHISPER = true,
		BLAME = true,
		CHANNEL = true,
	}

	local function outgoingFilter(self, event, msg)
		if msg:sub(1, 11) == "BigBrother>" then return true end
	end
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", outgoingFilter)

	local function getChannel(channel)
		channel = channel and channel:upper() or BigBrother.db.profile.outputDefault:upper()
		--seems like a dirty way to do this ;[ makes the syntax /bb qc channel:mychannel or /run BigBrother:ConsumableCheck("channel:mychannel")
		if channel and channel:sub(1, 7) == "CHANNEL" then
			local channelName = channel:sub(9)
			local index = GetChannelName(channelName)
			if index > 0 then
				return "CHANNEL", index
			else
				return
			end
		end
		if not channel or not chatTypes[channel] then
			return
		end

		if channel == "WHISPER" then
			if not UnitIsPlayer("target") then
				return
			end
			return channel, UnitName("target")
		elseif channel == "GROUP" then
			if IsInRaid() then
				channel = "RAID"
			elseif IsInGroup() then
				channel = "PARTY"
			else
				return
			end
		end
		if (channel == "RAID" or channel == "PARTY") and IsPartyLFG() then
			channel = "INSTANCE_CHAT"
		end

		return channel
	end

	function BigBrother:SendChatMessage(text, channel)
		local target
		channel, target = getChannel(channel)
		if not channel then return end
		if channel == "BLAME" then channel = "SELF" end

		if channel == "SELF" then
			self:Print(text)
		else
			SendChatMessage(text, channel, nil, target)
		end
	end

	function BigBrother:SendChatMessageList(title, list, channel)
		table.sort(list)
		local names = table.concat(list, ", ")

		local target
		channel, target = getChannel(channel)
		if not channel then return end

		if channel == "SELF" then
			self:Print(("|cffff8040%s (%d)|r: %s"):format(title, #list, names))
		else
			if channel ~= "BLAME" then
				SendChatMessage(("%s (%d): %s"):format(title, #list, names), channel, nil, target)
			else
				for _, name in next, list do
					ChatThrottleLib:SendChatMessage("BULK", "", ("BigBrother> %s"):format(title), "WHISPER", nil, name)
				end
			end
		end
	end
end

do -- BigBrother:GetCheckedGroups()
	local fiveManGroups = { true }
	local tenManGroups = { true, true }
	local twentyFiveManGroups = { true, true, true, true, true }
	local allGroups = { true, true, true, true, true, true, true, true }

	function BigBrother:GetCheckedGroups()
		if self.db.profile.autogroups then
			local size = self:GetInstanceSize()
			if size == 25 then
				return twentyFiveManGroups
			elseif size == 10 then
				return tenManGroups
			elseif size == 5 then
				return fiveManGroups
			else
				return allGroups
			end
		else
			return self.db.profile.groups
		end
	end
end

do --BigBrother:IterateGroup()
	local partyUnits = {"player"}
	for i = 1, MAX_PARTY_MEMBERS do
		partyUnits[#partyUnits + 1] = ("party%d"):format(i)
	end

	local raidUnits = {}
	for i = 1, MAX_RAID_MEMBERS do
		raidUnits[#raidUnits + 1] = ("raid%d"):format(i)
	end

	function BigBrother:IterateGroup()
		local i = 0
		local function iter(a)
			i = i + 1
			return a[i]
		end

		return iter, (IsInRaid() and raidUnits or partyUnits)
	end
end

function BigBrother:QuickCheck(channel)
	self:ConsumableCheck(channel)
	if self.db.profile.checkBuffs then
		self:BuffCheck(channel)
	end
end

function BigBrother:READY_CHECK(_, sender)
	-- unchecked (0), checked (1), greyed (2)
	if not IsInGroup() or IsPartyLFG() or self.db.profile.checkReadyCheck == 0 then return end

	if (self.db.profile.checkReadyCheck == 1 and UnitIsUnit(sender, "player")) or self.db.profile.checkReadyCheck == 2 then
		self:QuickCheck()
	end
end
